#
# $Id: demo.tcl,v 1.29 2004/01/23 07:12:59 jenglish Exp $
#
# Demo for 'tile' package.
#

lappend auto_path . [file dirname [info script]]

package require tile
package require tile::blue

# The descriptive names of the builtin themes.
set ::THEMELIST {
    {}       "Classic"
    alt      "Alternate"
    step     "OpenStep"
    winnative "Windows native"
    xpnative "XP Native"
}
array set ::THEMES $THEMELIST;

# Add in any available loadable themes.
foreach pkg [package names] {
    if {![string match tile::* $pkg]} { continue }
    set name [lindex [split $pkg :] end]
    if {$name eq "pixmap"} {
    	# not a real theme
	continue
    }
    if {![info exists ::THEMES($name)]} {
	lappend THEMELIST $name [set ::THEMES($name) [string totitle $name]]
    }
}

# This permits delayes loading of pixmap themes. So we dont load
# lots of images until we need them.
#
proc SetTheme {container style} {
    if {$style != {} && [package provide tile::$style] == {}} {
        if {[catch {package require tile::$style} msg]} {
            $container.s$style state disabled
            return -code error $msg
        }
    }
    style settheme $style
}

#
# Load icons...
#
proc loadIcons {file} {
    set ::BUTTONS [list open new save]
    set ::CHECKBOXES [list bold italic]

    source $file
    foreach icon [concat $::BUTTONS $::CHECKBOXES]  {
	set ::ICON($icon) [image create photo -data $ImgData($icon)]
    }
}
loadIcons [file join [file dirname [info script]] iconlib.tcl]

#
# Utilities:
#
proc foreachWidget {varname Q script} {
    upvar 1 $varname w
    while {[llength $Q]} {
    	set QN [list]
	foreach w $Q {
	    uplevel 1 $script
	    foreach child [winfo children $w] {
		lappend QN $child
	    }
	}
	set Q $QN
    }
}

# sbstub $sb --
#	Used as the -command option for a scrollbar,
#	updates the scrollbar's position.
#
proc sbstub {sb cmd number {units units}} { sbstub.$cmd $sb $number $units }
proc sbstub.moveto {sb number _} { $sb set $number [expr {$number + 0.5}] }
proc sbstub.scroll {sb number units} {
    if {$units eq "pages"} {
    	set delta 0.2
    } else {
	set delta 0.05
    }
    set current [$sb get]
    set new0 [expr {[lindex $current 0] + $delta*$number}]
    set new1 [expr {[lindex $current 1] + $delta*$number}]
    $sb set $new0 $new1
}

# ... for debugging:
bind all <ButtonPress-3> { set ::W %W }
bind all <Control-ButtonPress-3> { focus %W }

proc showHelp {} {
    tk_messageBox -message "No help yet..."
}

#
# The following demonstrates the flexibility of dynamic resources:
# [@@@@ AND IS CURRENTLY BROKEN -- IGNORE THIS BIT]
# Toolbar buttons normally have a flat relief, but "pop up"
# when the mouse is over them (the same effect you get with
# '-relief flat -overrelief raised' with Tk's regular buttons)
#
# In addition, with dynamic resources we can also specify different
# foreground and background colors for when the button is pressed,
# (without having to write a TIP to add new widget options!)
#
# For now, do this with the option database.  Eventually there
# will be an easier way to define customized widget styles so
# you can define this in one place and just say "button .b -style toolbar"
# to get the desired effect.
# [@@@@ WORKING ON THAT RIGHT NOW @@@@]
#
option add *Toolbar.relief groove
option add *Toolbar.borderWidth 2

option add *Toolbar.TButton.width 0
option add *Toolbar.TButton.padding 1
option add *Toolbar.TButton.relief flat
option add *Toolbar*font ToolbarFont

option add *Toolbar.Button.Pad 2

catch {font create ToolbarFont}
switch $tcl_platform(platform) {
    windows {
	font configure ToolbarFont -family Tahoma -size 8 -weight normal
    }
    unix -
    default {
	font configure ToolbarFont -family Helvetica -size 10 -weight normal
    }
}

set ::ROOT "."
set ::BASE ""
eval destroy [winfo children .]

array set ::V {
    STYLE	{}
    COMPOUND	top
    MENURADIO1	One
    MENUCHECK1	1
}

#
# Toolbar button standard vs. tile comparison:
#
proc makeToolbars {} {
    variable top
    set tb [frame $::BASE.tbar_styled -class Toolbar]
    set i 0
    foreach icon $::BUTTONS {
	set b [tbutton $tb.tb[incr i] \
		   -text $icon -image $::ICON($icon) -compound $::V(COMPOUND)]
	grid $b -row 0 -column $i -sticky news
    }
    foreach icon $::CHECKBOXES {
	set b [tcheckbutton $tb.cb[incr i] -variable ::V($icon) \
		   -text $icon -image $::ICON($icon) -compound $::V(COMPOUND)]
	grid $b -row 0 -column $i -sticky news
    }
    grid columnconfigure $tb [incr i] -weight 1

    set tb [frame $::BASE.tbar_orig -class Toolbar]
    set i 0
    foreach icon $::BUTTONS {
	set b [button $tb.tb[incr i] \
		  -text $icon -image $::ICON($icon) -compound $::V(COMPOUND) \
		  -relief flat -overrelief raised]
	grid $b -row 0 -column $i -sticky news
    }
    foreach icon $::CHECKBOXES {
	set b [checkbutton $tb.cb[incr i] -variable ::V($icon) \
		  -text $icon -image $::ICON($icon) -compound $::V(COMPOUND) \
		  -indicatoron false -selectcolor {}]
	grid $b -row 0 -column $i -sticky news
    }
    grid columnconfigure $tb [incr i] -weight 1
}
makeToolbars

## CONTROLS
frame $::BASE.control

#
# Overall theme control:
#
proc makeThemeControl {f} {
    set c [labelframe $f.style -text "Theme"]
    grid $c -sticky news -padx 6 -ipadx 6
    foreach {theme name} $::THEMELIST {
	set b [tradiobutton $c.s$theme -text $name -anchor w \
		   -variable ::V(STYLE) -value $theme \
		   -command [list SetTheme $c $theme]]
	grid $b -sticky ew
    }
    grid columnconfigure $c 0 -weight 1
    if {[package provide tile::xpnative] == {}} {
	$c.sxpnative state disabled
    }
}
makeThemeControl $::BASE.control

#
# Toolbar -compound control:
#
proc makeCompoundControl {f} {
    variable compoundStrings {text image none top bottom left right center}
    set c [labelframe $f.compound -text "Compound"]
    grid $c -sticky news -padx 6 -ipadx 6
    foreach string $compoundStrings {
	set b [tradiobutton $c.$string -anchor w \
		   -text [string totitle $string] \
		   -variable ::V(COMPOUND) -value $string \
		   -command changeToolbars]
	grid $b -sticky ew
    }
    grid columnconfigure $c 0 -weight 1
}
makeCompoundControl $::BASE.control

proc changeToolbars {} {
    foreachWidget w [list $::BASE.tbar_styled $::BASE.tbar_orig] {
	catch { $w configure -compound $::V(COMPOUND) }
    }
}

proc ScrolledWidget {parent class themed args} {
    if {$themed} {
        set sbcmd tscrollbar
    } else {
        set sbcmd scrollbar
    }

    for {set n 0} {[winfo exists $parent.f$n]} {incr n} {}
    set f [frame $parent.f$n]
    set t [eval [linsert $args 0 $class $f.$class]]
    set vs [$sbcmd $f.vs -orient vertical -command [list $t yview]]
    set hs [$sbcmd $f.hs -orient horizontal -command [list $t xview]]
    $t configure -yscrollcommand [list $vs set] -xscrollcommand [list $hs set]

    grid configure $t $vs -sticky news
    grid configure $hs x  -sticky news
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1

    return $f
}

#
# Notebook demonstration:
#
proc makeNotebook {} {
    set nb [tnotebook $::BASE.nb]
    set client [frame $nb.client]
    $nb add $client -text "Demo"
    $nb raise $client

    $nb add [tlabel $nb.stuff -text "Nothing to see here..."] \
	-text "Stuff"
    $nb add [tlabel $nb.more -text "Nothing to see here either."] \
	-text "More stuff"

    return $client
}
set client [makeNotebook]

#
# Side-by side check & radiobutton comparison:
#

set l [labelframe $client.styled -text "Styled"]
set r [labelframe $client.orig -text "Standard"]

set ltext [ScrolledWidget $l text 1 -width 12 -height 5 -wrap none]
set rtext [ScrolledWidget $r text 0 -width 12 -height 5 -wrap none]

set msg "The cat crept into the crypt, crapped and crept out again\n"
for {set n 0} {$n < 50} {incr n} {
    $ltext.text insert end "$n: $msg"
    $rtext.text insert end "$n: $msg"
}

## Styled frame
tcheckbutton $l.cb -text "Checkbutton" -variable ::V(SELECTED) -underline 2
tradiobutton $l.rb1 -text "One" -variable ::V(CHOICE) -value 1 -underline 0
tradiobutton $l.rb2 -text "Two" -variable ::V(CHOICE) -value 2
tradiobutton $l.rb3 -text "Three" -variable ::V(CHOICE) -value 3 -under 0
tbutton $l.button -text "Button" -underline 0
tscale $l.scale -orient horizontal -from 0 -to 100 -variable ::V(SCALE)
tscale $l.vscale -orient vertical -from -25 -to 25 -variable ::V(VSCALE)

# NOTE TO MAINTAINERS: 
# The checkbuttons are -sticky ew / -expand x  on purpose:
# it demonstrates one of the differences between TCheckbuttons
# and standard checkbuttons.
#
grid $l.cb  -sticky ew
grid $l.rb1 -sticky ew
grid $l.rb2 -sticky ew
grid $l.rb3 -sticky ew
grid $l.button -sticky ew -padx 2 -pady 2
grid $ltext -sticky news
grid $l.scale -sticky news
grid $l.vscale -sticky nws

grid columnconfigure $l 0 -weight 1
grid rowconfigure    $l 5 -weight 1 ; # text widget

if {[llength [info command tprogress]]} {
    grid forget $l.vscale
    frame $l.vp
    tprogress $l.progress -orient horizontal -from 0 -to 100
    tprogress $l.vprogress -orient vertical -from -25 -to 25
    grid $l.progress -sticky news
    grid $l.vp -sticky nws
    grid $l.vscale $l.vprogress -in $l.vp -sticky ns
    raise $l.vscale
    $l.scale configure -command [list $l.progress set]
    $l.progress set [$l.progress cget -from]
    $l.vscale configure -command [list $l.vprogress set]
    $l.vprogress set [$l.vprogress cget -from]
}

## Orig frame
checkbutton $r.cb -text "Checkbutton" -variable ::V(SELECTED) 
radiobutton $r.rb1 -text "One" -variable ::V(CHOICE) -value 1 
radiobutton $r.rb2 -text "Two" -variable ::V(CHOICE) -value 2 -underline 1
radiobutton $r.rb3 -text "Three" -variable ::V(CHOICE) -value 3
button $r.button -text "Button"
scale $r.scale -orient horizontal -from 0 -to 100 -variable ::V(SCALE)
scale $r.vscale -orient vertical -from -25 -to 25 -variable ::V(VSCALE)

grid $r.cb -sticky ew
grid $r.rb1 -sticky ew
grid $r.rb2 -sticky ew
grid $r.rb3 -sticky ew
grid $r.button -sticky ew -padx 2 -pady 2
grid $rtext -sticky news
grid $r.scale -sticky news
grid $r.vscale -sticky nws

grid columnconfigure $r 0 -weight 1
grid rowconfigure    $r 5 -weight 1 ; # text widget

grid $client.styled $client.orig -sticky news -padx 6 -pady 6
grid rowconfigure $client 0 -weight 1
grid columnconfigure $client {0 1} -weight 1

#
# Command box:
#
set cmd [frame $::BASE.command]
tbutton $cmd.close -text Close -underline 0 -default normal \
    -command [list destroy .]
tbutton $cmd.help -text Help -underline 0 -default normal \
    -command showHelp

grid $cmd.close $cmd.help -pady {6 4} -padx 4

#
# Set up accelerators:
#
bind $::ROOT <KeyPress-Escape>	[list event generate $cmd.close <<Invoke>>]
bind $::ROOT <<Help>>		[list event generate $cmd.help <<Invoke>>]
keynav::enableMnemonics $::ROOT
keynav::defaultButton $cmd.help

grid $::BASE.tbar_styled - -sticky ew
grid $::BASE.tbar_orig   - -sticky ew
grid $::BASE.control     $::BASE.nb -sticky news
grid $::BASE.command     - -sticky e
grid configure $::BASE.control -sticky new
grid columnconfigure $::ROOT 1 -weight 1
grid rowconfigure    $::ROOT 2 -weight 1

#
# Add a menu
#
set menu [menu $::BASE.menu]
$::ROOT configure -menu $menu
$menu add cascade -label "File" -underline 0 \
    -menu [menu $menu.file -tearoff 0]
$menu.file add command -label "Open" -underline 0 \
    -compound left -image $::ICON(open)
$menu.file add command -label "Save" -underline 0 \
    -compound left -image $::ICON(save)
$menu.file add separator
$menu.file add cascade -label "Test submenu" -underline 0 \
    -menu [menu $menu.file.test -tearoff 0]
$menu.file add checkbutton -label "Text check" -underline 5 \
    -variable ::V(MENUCHECK1)
$menu.file add command -label "Exit" -underline 1 \
    -command [list event generate $cmd.close <<Invoke>>]

foreach label {One Two Three Four} {
    $menu.file.test add radiobutton -label $label -variable ::V(MENURADIO1)
}

# Add a theme menu.
# This uses the package command to dynamically build a list of available
# themes.
proc settheme {style} {
    style settheme $style
    set ::V(STYLE) $style
}
$menu add cascade -label "Theme" -underline 3 \
    -menu [menu $menu.theme -tearoff 0]
foreach {theme name} $::THEMELIST {
    $menu.theme add radiobutton -label $name \
        -variable ::V(STYLE) -value $theme -command [list settheme $theme]
}

#
# Add a console menu item for windows.
#

if {[tk windowingsystem] == "win32"} {
    set console 0
    proc toggleconsole {} {
        if {$::V(CONSOLE)} {console show} else {console hide}
    }

    set ndx [$menu.file index end]
    $menu.file insert $ndx checkbutton -label Console -underline 0 \
        -variable ::V(CONSOLE) -command toggleconsole
}

settheme $::V(STYLE)
