#
# $Id: tile.tcl,v 1.47 2004/01/23 01:42:08 jenglish Exp $
#
# Tile widget set initialization script. 
#

if {![info exists tile_library]} {
    set tile_library [file dirname [info script]]
}

lappend auto_path $tile_library

source [file join $tile_library keynav.tcl]
source [file join $tile_library button.tcl]
source [file join $tile_library scrollbar.tcl]
source [file join $tile_library scale.tcl]
source [file join $tile_library notebook.tcl]

#
# Helper routines:
#
namespace eval Tile {
    proc takefocus {w} {
	return [$w instate !disabled]
    }
    proc activate {w} {
	set oldState [$w state pressed]
	update ; after 100
	$w state $oldState
	after idle [list $w invoke]
    }

    proc arm {w} {
	$w state pressed
	event generate $w <<Arm>>
    }
    proc disarm {w} {
	$w state !pressed
	event generate $w <<Disarm>>
    }

    #
    # Routines for auto-repeat:
    #
    # NOTE: repeating widgets do not have -repeatdelay
    # or -repeatinterval resources as in standard Tk;
    # instead a single set of settings is applied application-wide.
    # (TODO: make this user-configurable)
    #
    # (@@@ Windows seems to use something like 500/50 millisecons
    #  @@@ for -repeatdelay/-repeatinterval)
    #

    variable Repeat
    array set Repeat {
	    delay	300
	    interval	100
	    timer 	{}
	    script	{}
    }

    proc Repeatedly {args} {
	variable Repeat
	after cancel $Repeat(timer)
	set script [uplevel 1 [list namespace code $args]]
	set Repeat(script) $script
	uplevel #0 $script
	set Repeat(timer) [after $Repeat(delay) Tile::Repeat]
    }

    proc Repeat {} {
	variable Repeat
	uplevel #0 $Repeat(script)
	set Repeat(timer) [after $Repeat(interval) Tile::Repeat]
    }

    proc CancelRepeat {} {
	variable Repeat
	after cancel $Repeat(timer)
    }

    #
    # ThemeChanged --
    #	Called from [style settheme].
    #	Notifies all widgets that the theme has changed.
    #	
    proc ThemeChanged {} {
	set Q .
	while {[llength $Q]} {
	    set QN [list]
	    foreach w $Q {
		catch { $w configure -style [$w cget -style] }
		foreach child [winfo children $w] {
		    lappend QN $child
		}
	    }
	    set Q $QN
	}
    }

}

#
# Appearance:
#
catch {font create ButtonFont}

namespace eval Tile {

    switch $tcl_platform(platform) {
    windows {
    	font configure ButtonFont -family {MS Sans Serif} -size 8 -weight normal
	set defaultBG "SystemButtonFace"
	set activeBG  "SystemButtonFace"
	set troughColor "SystemScrollbar"
    }
    unix -
    default {
	font configure ButtonFont -family Helvetica -size -12 -weight bold
	set defaultBG #d9d9d9
	set activeBG  #ececec
	set troughColor #c3c3c3
    } }
}

option add *TButton.font		ButtonFont
option add *TCheckbutton.font		ButtonFont
option add *TRadiobutton.font		ButtonFont
option add *TLabel.font			ButtonFont
option add *TScrollbar.takeFocus	0


#
# Load themes:
#
source [file join $tile_library defaults.tcl]
source [file join $tile_library altTheme.tcl]
source [file join $tile_library stepTheme.tcl]

if {[package provide tile::winnative] != {}} {
    source [file join $tile_library winTheme.tcl]
}
if {[package provide tile::xpnative] != {}} {
    source [file join $tile_library xpTheme.tcl]
}

#*EOF*
